# -*- coding: utf-8 -*-

"""
Created on 2015-12-16
:author: Andreas Kaiser (disko@binary-punks.com)
"""
from pyramid.view import view_config

from kotti.views.edit.content import FileEditForm, FileAddForm

from kotti_image import _
from kotti_image.resources import Image


@view_config(name=Image.type_info.add_view,
             permission=Image.type_info.add_permission,
             renderer='kotti:templates/edit/node.pt')
class ImageAddForm(FileAddForm):
    item_type = _(u"Image")
    item_class = Image


@view_config(name='edit', context=Image, permission='edit',
             renderer='kotti:templates/edit/node.pt')
class ImageEditForm(FileEditForm):
    pass



import pytest
from webtest import Upload


@pytest.fixture
def gallery(root):
    from kotti.resources import Document
    root['my-gallery'] = Document(title=u'My Gallery')


def asset(name):
    import kotti_image
    from os.path import dirname
    from os.path import join
    return open(join(dirname(kotti_image.__file__), 'tests', name), 'rb')


@pytest.fixture
def assets():

    return {
        'img1': {
            'filename': 'sendeschluss.jpg',
            'content': asset('sendeschluss.jpg').read(),
            'content_type': 'image/jpeg',
        },
        'img2': {
            'filename': 'logo.png',
            'content': asset('logo.png').read(),
            'content_type': 'image/png',
        },
    }


@pytest.mark.user('admin')
def test_add_images(webtest, gallery, assets):

    # add img 1
    resp = webtest.get('/my-gallery')
    resp = resp.click(href='add_image')
    form = resp.forms['deform']
    form['title'] = 'Image 1'
    form['description'] = 'A beautiful image'
    form['upload'] = Upload(
        assets['img1']['filename'],
        assets['img1']['content'],
        assets['img1']['content_type'])
    resp = form.submit('save').maybe_follow()
    assert b'Item was added' in resp.body

    # add img 2
    resp = webtest.get('/my-gallery')
    resp = resp.click(href='add_image')
    form = resp.forms['deform']
    form['title'] = 'Image 2'
    form['description'] = 'An even more beautiful image'
    form['upload'] = Upload(
        assets['img2']['filename'],
        assets['img2']['content'],
        assets['img2']['content_type'])
    resp = form.submit('save').maybe_follow()
    assert b'Item was added' in resp.body

    # img without file
    resp = webtest.get('/my-gallery')
    resp = resp.click(href='add_image')
    form = resp.forms['deform']
    form['title'] = 'Image 2'
    form['description'] = 'An even more beautiful image'
    resp = form.submit('save').maybe_follow()
    assert b'There was a problem with your submission' in resp.body
    assert b'Required' in resp.body

    # View gallery and images
    resp = webtest.get('/my-gallery/@@contents')
    assert b'Image 1' in resp.body
    assert b'Image 2' in resp.body
    assert resp.body.find(b'Image 1') < resp.body.find(b'Image 2')
    resp = webtest.get('/my-gallery/image-1')
    assert resp.content_type == 'text/html'
    resp = webtest.get('/my-gallery/image-2')
    assert resp.content_type == 'text/html'

    #
    # "OLD STYLE" image scales (will be removed in 1.0.0)
    #

    # Default scale
    resp = webtest.get('/my-gallery/image-1/image')
    assert resp.content_type == assets['img1']['content_type']
    assert resp.content_length == len(assets['img1']['content'])
    assert resp.content_disposition == 'inline; filename={0}'.format(
        assets['img1']['filename'])
    resp = webtest.get('/my-gallery/image-2/image')
    assert resp.content_type == assets['img2']['content_type']
    assert resp.content_length == len(assets['img2']['content'])
    assert resp.content_disposition == 'inline; filename={0}'.format(
        assets['img2']['filename'])

    # Default scale, attachment
    resp = webtest.get('/my-gallery/image-1/image/download')
    assert resp.content_type == assets['img1']['content_type']
    assert resp.content_length == len(assets['img1']['content'])
    assert resp.content_disposition == 'attachment; filename={0}'.format(
        assets['img1']['filename'])
    resp = webtest.get('/my-gallery/image-2/image/download')
    assert resp.content_type == assets['img2']['content_type']
    assert resp.content_length == len(assets['img2']['content'])
    assert resp.content_disposition == 'attachment; filename={0}'.format(
        assets['img2']['filename'])

    # span1
    resp = webtest.get('/my-gallery/image-1/image/span1')
    assert resp.content_type == assets['img1']['content_type']
    assert 1000 < resp.content_length < 6000
    assert resp.content_disposition == 'inline; filename={0}'.format(
        assets['img1']['filename'])

    # span1, attachment
    resp = webtest.get('/my-gallery/image-1/image/span1/download')
    assert resp.content_type == assets['img1']['content_type']
    assert 1000 < resp.content_length < 6000
    assert resp.content_disposition == 'attachment; filename={0}'.format(
        assets['img1']['filename'])

    # Invalid predefined scale (should return original size)
    resp = webtest.get('/my-gallery/image-1/image/invalid_scale')
    assert resp.content_type == assets['img1']['content_type']
    assert resp.content_length == len(assets['img1']['content'])
    assert resp.content_disposition == 'inline; filename={0}'.format(
        assets['img1']['filename'])

    # Invalid predefined scale , attachment (should return original size)
    resp = webtest.get('/my-gallery/image-1/image/invalid_scale/download')
    assert resp.content_type == assets['img1']['content_type']
    assert resp.content_length == len(assets['img1']['content'])
    assert resp.content_disposition == 'attachment; filename={0}'.format(
        assets['img1']['filename'])


class TestTween:

    @pytest.mark.user('admin')
    def test_tween(self, connection, webtest, filedepot, root, image_asset,
                   db_session):

        from kotti_image.resources import Image
        from kotti.resources import get_root

        # create an image resource
        root['img'] = Image(data=image_asset.read(), title=u'Image')
        db_session.flush()
        root = get_root()
        img = root['img']

        # the image resource itself is served by the full Kotti stack
        resp = webtest.get('/img')
        assert resp.content_type == 'text/html'
        assert resp.etag is None
        assert resp.cache_control.max_age == 0
        assert u'<img src="http://localhost/img/image" />' in resp.text

        # the attachments (created by the filters) are served by the
        # FiledepotServeApp.
        resp = webtest.get(img.data[u'thumb_128x128_url'])

        assert resp.etag is not None
        assert resp.cache_control.max_age == 604800
        assert resp.body.startswith(b'\x89PNG')

        resp = webtest.get(img.data[u'thumb_256x256_url'])
        assert resp.etag is not None
        assert resp.cache_control.max_age == 604800
        assert resp.body.startswith(b'\x89PNG')
        resp = webtest.get(img.data[u'thumb_256x256_url'] + 'not', status=404)
        assert resp.status_code == 404



# -*- coding: utf-8 -*-

"""
Created on 2015-12-16
:author: Andreas Kaiser (disko@binary-punks.com)
"""

pytest_plugins = "kotti"

from pytest import fixture


@fixture(scope='session')
def custom_settings():
    # Compatibility for Kotti 1.3 and 2.0
    try:
        from kotti.resources import Image
        return {}
    except ImportError:
        # noinspection PyUnresolvedReferences c
        from kotti_image.resources import Image
        return {
            'kotti.configurators': 'kotti_image.kotti_configure'
        }



# -*- coding: utf-8 -*-

"""
Created on 2015-12-16
:author: Andreas Kaiser (disko@binary-punks.com)
"""
from depot.fields.filters.thumbnails import WithThumbnailFilter
from kotti.resources import Content
from kotti.resources import SaveDataMixin
from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from zope.interface import implementer

from kotti_image import _
from kotti_image.interfaces import IImage


@implementer(IImage)
class Image(SaveDataMixin, Content):
    """Image is a specialized version of :class:`~kotti.resources.File`, that
    adds thumbnails and has different views.
    """

    #: Primary key column in the DB
    #: (:class:`sqlalchemy.types.Integer`)
    id = Column(Integer(), ForeignKey('contents.id'), primary_key=True)

    data_filters = (
        WithThumbnailFilter(size=(128, 128), format='PNG'),
        WithThumbnailFilter(size=(256, 256), format='PNG'))

    type_info = Content.type_info.copy(
        name=u'Image',
        title=_(u'Image'),
        add_view=u'add_image',
        addable_to=[u'Document'],
        selectable_default_views=[],
        uploadable_mimetypes=['image/*', ],
        )



# -*- coding: utf-8 -*-

"""
Created on 2015-12-16
:author: Andreas Kaiser (disko@binary-punks.com)
"""

from pyramid.i18n import TranslationStringFactory

_ = TranslationStringFactory('kotti_image')


def kotti_configure(settings):
    """ Add a line like this to you .ini file::

            kotti.configurators =
                kotti_image.kotti_configure

        to enable the ``kotti_image`` add-on.

    :param settings: Kotti configuration dictionary.
    :type settings: dict
    """

    settings['pyramid.includes'] += ' kotti_image'
    settings['kotti.alembic_dirs'] += ' kotti_image:alembic'
    settings['kotti.available_types'] += ' kotti_image.resources.Image'


def includeme(config):
    """ Don't add this to your ``pyramid_includes``, but add the
    ``kotti_configure`` above to your ``kotti.configurators`` instead.

    :param config: Pyramid configurator object.
    :type config: :class:`pyramid.config.Configurator`
    """

    config.add_translation_dirs('kotti_image:locale')
    # config.add_static_view('static-kotti_image', 'kotti_image:static')

    config.scan(__name__)



import os
import sys

from setuptools import setup
from setuptools import find_packages

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''
try:
    CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()
except IOError:
    CHANGES = ''

version = '1.0.1'

if sys.version_info >= (3, ):
    install_requires = ['Kotti>=2.0.0.dev0', ]
else:
    install_requires = ['Kotti<2.0.0.dev0', ]

install_requires += [
    'Pillow',  # dependency of plone.scale
    'plone.scale',  # needed for image resizing capabilities
    'rfc6266-parser',
    'unidecode',
]

# copied from Kotti, necessary because extras are not supported in
# ``extras_require``.  See https://github.com/pypa/pip/issues/3189
tests_require = [
    'WebTest',
    'mock',
    'Pillow',  # thumbnail filter in depot tween tests
    'py>=1.4.29',
    'pyquery',
    'pytest>=3.0.0',
    'pytest-cov',
    'pytest-pep8!=1.0.3',
    'pytest-travis-fold',
    'pytest-virtualenv',
    'pytest-xdist',
    'tox',
    'zope.testbrowser>=5.0.0',
    ]

# copied from Kotti, necessary because extras are not supported in
# ``extras_require``.  See https://github.com/pypa/pip/issues/3189
development_requires = [
    'check-manifest',
    'pipdeptree',
    'pyramid_debugtoolbar',
]

setup_requires = [
    'setuptools_git>=0.3',
]


setup(
    name='kotti_image',
    version=version,
    description="Image content type for Kotti",
    long_description='\n\n'.join([README, CHANGES]),
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: Implementation :: CPython',
        "Framework :: Pylons",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "License :: Repoze Public License",
    ],
    author='Kotti Developers',
    author_email='kotti@googlegroups.com',
    url='https://github.com/Kotti/kotti_image',
    keywords='kotti web cms wcms pylons pyramid sqlalchemy bootstrap',
    license="BSD-derived (http://www.repoze.org/LICENSE.txt)",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    setup_requires=setup_requires,
    dependency_links=[],
    entry_points={},
    extras_require={
        'testing': tests_require,
        'development': development_requires,
    },
)



# -*- coding: utf-8 -*-

"""
Created on 2015-12-16
:author: Andreas Kaiser (disko@binary-punks.com)
"""

import PIL
import rfc6266_parser
from kotti.util import extract_from_settings
from plone.scale.scale import scaleImage
from pyramid.response import Response
from pyramid.view import view_config
from pyramid.view import view_defaults
from unidecode import unidecode

from kotti_image.interfaces import IImage

PIL.ImageFile.MAXBLOCK = 33554432

#: Default image scales
image_scales = {
    'span1': [60, 120],
    'span2': [160, 320],
    'span3': [260, 520],
    'span4': [360, 720],
    'span5': [460, 920],
    'span6': [560, 1120],
    'span7': [660, 1320],
    'span8': [760, 1520],
    'span9': [860, 1720],
    'span10': [960, 1920],
    'span11': [1060, 2120],
    'span12': [1160, 2320],
    }


@view_defaults(context=IImage, permission='view')
class ImageView(object):
    """The ImageView class is registered for the :class:`IImage` context."""

    def __init__(self, context, request):

        self.context = context
        self.request = request

    @view_config(name='view', renderer='kotti_image:templates/image.pt')
    def view(self):
        """
        :result: Empty dictionary to be handed to the image.pt template for
                 rendering.
        :rtype: dict
        """

        return {}

    @view_config(name='image')
    def image(self, subpath=None):
        """Return the image in a specific scale, either inline
        (default) or as attachment.

        :param subpath: [<image_scale>]/download] (optional).
                        When 'download' is the last element in subpath,
                        the image is served with a 'Content-Disposition:
                        attachment' header.  <image_scale> has to be one of the
                        predefined image_scales - either from the defaults in
                        this module or one set with a
                        kotti.image_scales.<scale_name> in your app config ini
                        file.
        :type subpath: str

        :result: complete response object
        :rtype: pyramid.response.Response
        """

        if subpath is None:
            subpath = self.request.subpath

        width, height = (None, None)
        subpath = list(subpath)

        if (len(subpath) > 0) and (subpath[-1] == 'download'):
            disposition = 'attachment'
            subpath.pop()
        else:
            disposition = 'inline'

        if len(subpath) == 1:
            scale = subpath[0]
            if scale in image_scales:
                # /path/to/image/scale/thumb
                width, height = image_scales[scale]

        if not (width and height):
            return self.request.uploaded_file_response(
                self.context.data, disposition)

        image, format, size = scaleImage(self.context.data.file.read(),
                                         width=width,
                                         height=height,
                                         direction='thumb')
        res = Response(
            headerlist=[
                ('Content-Disposition', '{0};filename="{1}"'.format(
                    disposition,
                    self.context.filename.encode('ascii', 'ignore'))),
                ('Content-Length', str(len(image))),
                ('Content-Type', str(self.context.mimetype)),
            ],
            body=image,
        )
        res.content_disposition = rfc6266_parser.build_header(
            self.context.filename, disposition=disposition,
            filename_compat=unidecode(self.context.filename))

        return res


def _load_image_scales(settings):
    image_scale_strings = extract_from_settings('kotti.image_scales.', settings)

    for k in image_scale_strings.keys():
        image_scales[k] = [int(x) for x in image_scale_strings[k].split('x')]


def includeme(config):
    _load_image_scales(config.registry.settings)

    config.scan(__name__)





def test_image_scale_loading_it():

    from kotti_image.views import _load_image_scales, image_scales

    assert "daumennagel" not in image_scales

    _load_image_scales({"kotti.image_scales.daumennagel": "100x100"})

    assert image_scales["daumennagel"] == [100, 100]
